@@ -15,19 +15,6 @@ sub new
 {
 	shift;
 	my $connect_line= shift;
-
-# [R] self-hack to allow empty UID and PWD
-	my $temp_connect_line;
-	$connect_line=~/DSN=\w+/;
-	$temp_connect_line="$&;";
-	if ($connect_line=~/UID=\w?/)
-		{$temp_connect_line.="$&;";}
-	else	{$temp_connect_line.="UID=;";};
-	if ($connect_line=~/PWD=\w?/)
-		{$temp_connect_line.="$&;";}
-	else	{$temp_connect_line.="PWD=;";};
-	$connect_line=$temp_connect_line;
-# -[R]-
 	
 	my $self= {};
 		
@@ -96,9 +83,6 @@ sub Sql
 		$self->{'DBI_NAME'} = $sth->{NAME};
 	}
 
-# [R] provide compatibility with Win32::ODBC's way of identifying erroneous SQL statements
- 	return ($self->{'DBI_ERR'})?1:undef;
-# -[R]-
 }
  
 
@@ -110,32 +94,17 @@ sub FetchRow
  	my $sth=$self->{'DBI_STH'};
  	if ($sth)
 	{
-	 	my @row=$sth->fetchrow_array;
+	 	my @row=$sth->fetchrow;
 	 	$self->{'DBI_ROW'}=\@row;
 
 	 	if (scalar(@row)>0)
 	 	{
 			#-- the row of result is not nul
-			#-- return something nothing will be return else
-			return 1;
+			#-- return somthing nothing will be return else
+			1;
 	 	} 	
 	}
-	return undef;
 } 
-
-# [R] provide compatibility with Win32::ODBC's Data() method.
-sub Data
-{
-	my $self=shift;
-	my @array=@{$self->{'DBI_ROW'}};
-	foreach my $element (@array)
-	{
-		# remove padding of spaces by DBI
-		$element=~s/(\s*$)//;
-	};
-	return (wantarray())?@array:join('', @array);
-};
-# -[R]-
  
 #EMU --- %record = $db->DataHash;
 sub DataHash
@@ -149,36 +118,23 @@ sub DataHash
  	my @row=@$p_row;
 
  	my %DataHash;
-#print @name; print "\n"; print @row;
-# [R] new code that seems to work consistent with Win32::ODBC
-	while (@name)
+ 	my $name;
+ 	my $row;
+
+	foreach $name (@name)
 	{
-		my $name=shift(@name);
-		my $value=shift(@row);
-
-		# remove padding of spaces by DBI
-		$name=~s/(\s*$)//;
-		$value=~s/(\s*$)//;
-
-		$DataHash{$name}=$value;
-	};
-# -[R]-
-
-# [R] old code that didn't appear to work
-#	foreach my $name (@name)
-#	{
-#		$name=~s/(^\s*)|(\s*$)//;
-#		my @arr=@$name;
-#		foreach (@arr)
-#		{
-#			print "lot $name  name  col $_   or ROW= 0 $row[0]  1 $row[1] 2 $row[2] \n ";
-#			$DataHash{$name}=shift(@row);
-#		}
-#	}
-# -[R]-
+		my @arr=@$name;
+		foreach (@arr)
+		{
+			#print "lot $name  name  col $_   or ROW= 0 $row[0]  1 $row[1] 2 $row[2] \n ";
+			$DataHash{$_}=shift(@row);
+		}
+		
+		
+	}
 
  	#--- Return Hash
- 	return %DataHash; 	
+ 	%DataHash; 	
 } 
 
 
@@ -195,24 +151,13 @@ sub Error
 
  	#-- else good no error message 	
  	
-}
-
-# [R] provide compatibility with Win32::ODBC's Close() method.
-sub Close
-{
-	my $self=shift;
-
-	my $dbh=$self->{'DBI_DBH'};
-	$dbh->disconnect;
-}
-# -[R]-
+} 
 
+ 
 1;
 
 __END__
 
-# [R] to -[R]- indicate sections edited by me, Roy Lee
-
 =head1 NAME
 
 Win32::DBIODBC - Win32::ODBC emulation layer for the DBI
@@ -239,7 +184,7 @@ scripts by doing
 
 =head1 TO DO
 
-Error handling is virtually non-existent.
+Error handling is virtually non-existant.
 
 =head1 AUTHOR
 
